from datetime import datetime

import requests

from nicovideo_api_client.api.v2.targets import SnapshotSearchAPIV2Targets
from nicovideo_api_client.constants import END_POINT_URL_V2_VERSION, FieldType


class SnapshotSearchAPIV2:
    """
    `ニコニコ動画 『スナップショット検索API v2』 <https://site.nicovideo.jp/search-api-docs/snapshot>`_ のエンドポイント
    ここの関数から返るオブジェクトをメソッドチェーンをつないで呼び出すことで適切に設定された状態でAPIを叩くことができる。
    """
    @staticmethod
    def tags_exact() -> SnapshotSearchAPIV2Targets:
        """
        タグ検索を行う

        :return: クエリ(キーワード)入力オブジェクト
        """
        return SnapshotSearchAPIV2Targets(FieldType.TAGS_EXACT)

    @staticmethod
    def keywords() -> SnapshotSearchAPIV2Targets:
        """
        キーワード検索を行う

        :return: クエリ(キーワード)入力オブジェクト
        """
        return SnapshotSearchAPIV2Targets(FieldType.TITLE, FieldType.DESCRIPTION, FieldType.TAGS)

    @staticmethod
    def version(timeout: float = 400) -> datetime:
        """
        API の最終更新日時を取得する。

        `データの更新について <https://site.nicovideo.jp/search-api-docs/snapshot#
        %E3%83%87%E3%83%BC%E3%82%BF%E3%81%AE%E6%9B%B4%E6%96%B0%E3%81%AB%E3%81%A
        4%E3%81%84%E3%81%A6>`_ を参照。

        Note: 実際にリクエストが行われる。

        :param timeout: リクエストのタイムアウト時間。
        :return: 最終更新日時
        """
        json = requests.get(END_POINT_URL_V2_VERSION, timeout=timeout).json()
        return datetime.fromisoformat(json["last_modified"])



from enum import Enum


class FieldType(Enum):
    CONTENT_ID = 'contentId'
    TITLE = 'title'
    DESCRIPTION = 'description'
    VIEW_COUNTER = 'viewCounter'
    MYLIST_COUNTER = 'mylistCounter'
    LENGTH_SECONDS = 'lengthSeconds'
    THUMBNAIL_URL = 'thumbnailUrl'
    START_TIME = 'startTime'
    LAST_RES_BODY = 'lastResBody'
    COMMENT_COUNTER = 'commentCounter'
    LAST_COMMENT_TIME = 'lastCommentTime'
    CATEGORY_TAGS = 'categoryTags'
    TAGS = 'tags'
    TAGS_EXACT = 'tagsExact'
    GENRE = 'genre'
    GENRE_KEYWORD = 'genre.keyword'


END_POINT_URL_V2 = 'https://api.search.nicovideo.jp/api/v2/snapshot/video/contents/search'
END_POINT_URL_NVV3 = 'https://nvapi.nicovideo.jp/v3/'
END_POINT_URL_NVV2 = 'https://nvapi.nicovideo.jp/v2/'
END_POINT_URL_VIDEO = 'https://www.nicovideo.jp/watch/'
END_POINT_URL_V2_VERSION = 'https://api.search.nicovideo.jp/api/v2/snapshot/version'

sort_types = [
    FieldType.VIEW_COUNTER,
    FieldType.MYLIST_COUNTER,
    FieldType.LENGTH_SECONDS,
    FieldType.START_TIME,
    FieldType.COMMENT_COUNTER,
    FieldType.LAST_COMMENT_TIME
]


class SortKeyType(Enum):
    REGISTERED_AT = 'registeredAt'



from typing import Dict

from nicovideo_api_client.api.v2.request import SnapshotSearchAPIV2Request


class SnapshotSearchAPIV2Limit:

    def __init__(self, query: Dict[str, str]):
        self._query: Dict[str, str] = query

    def limit(self, limit: int = 10) -> SnapshotSearchAPIV2Request:
        """
        レスポンスの要素数を指定する。

        :param limit: レスポンスの要素数の上限。デフォルトは API 仕様に則り 10件。
        本来の件数上限は 100件だが、分割リクエストを送るためそれ以上を指定できる。
        :return: リクエストオブジェクト
        """
        return SnapshotSearchAPIV2Request(self._query, limit)



import re
from datetime import datetime
from typing import Dict, List, Any

from nicovideo_api_client.api.v2.limit import SnapshotSearchAPIV2Limit
from nicovideo_api_client.constants import FieldType


class SnapshotSearchAPIV2SimpleFilter:
    def __init__(self, query: Dict[str, str]):
        self._query: Dict[str, str] = query
        self._filters: Dict[str, List[Any]] = {}

    def set_filter(self, field_type: FieldType, value: Any):
        if field_type == FieldType.START_TIME:
            if not isinstance(value, datetime):
                raise TypeError("FieldType.START_TIMEを指定した時の型は datetime であるべきです")
            v = value.strftime('%Y-%m-%dT%H:%M:%S+09:00')
        elif field_type == FieldType.CONTENT_ID:
            if isinstance(value, int):
                v = f"sm{value}"
            elif isinstance(value, str):
                if not re.compile(r"sm\d+").fullmatch(value):
                    raise TypeError("FieldType.CONTENT_IDはsm(数字)の形で表されるべきです")
                v = value
            else:
                raise TypeError("FieldType.CONTENT_IDを指定した時の型は int または str であるべきです")
        else:
            raise NotImplementedError("未知のTypeが指定されました")

        if field_type.value not in self._filters:
            self._filters[field_type.value] = []

        self._filters[field_type.value].append(v)

        self._query[
            f'filters[{field_type.value}][{len(self._filters[field_type.value])}]'
        ] = v
        return self

    def filter(self) -> SnapshotSearchAPIV2Limit:
        return SnapshotSearchAPIV2Limit(self._query)



# -*- coding: utf-8 -*-

from setuptools import setup
from codecs import open
from os import path
import re

package_name = "nicovideo_api_client"

packages = [package_name, "nicovideo_api_client.api.v2"]

root_dir = path.abspath(path.dirname(__file__))


def _requirements():
    return [name.rstrip() for name in open(path.join(root_dir, 'requirements.txt')).readlines()]


with open(path.join(root_dir, package_name, '__init__.py')) as f:
    init_text = f.read()
    version = re.search(r'__version__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    # license = re.search(r'__license__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    author = re.search(r'__author__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    author_email = re.search(r'__author_email__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    url = re.search(r'__url__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)

assert version
# assert license
assert author
assert author_email
assert url

with open('README.rst', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name=package_name,
    packages=packages,

    version=version,

    # license=license,

    install_requires=_requirements(),

    author=author,
    author_email=author_email,

    url=url,

    description='ニコニコ動画 スナップショット検索APIv2の Python クライアント',
    long_description=long_description,

    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
    ],
)



from typing import Dict, List, Tuple
from urllib.parse import urlencode, unquote_plus

import math
import requests
import time

from nicovideo_api_client.api.v2.result import SnapshotSearchAPIV2Result
from nicovideo_api_client.constants import END_POINT_URL_V2


class SnapshotSearchAPIV2Request:

    def __init__(self, query: Dict[str, str], limit: int):
        self._query: Dict[str, str] = query
        self._limit = limit

    def request(self, timeout: float = 400.0) -> SnapshotSearchAPIV2Result:
        """
        API にリクエストを送り、レスポンスを返す。

        `limit()` で 100件を超える値が設定されている場合、複数回のリクエストに分けて実行される。
        (そのため、 `timeout` の設定には注意が必要)

        :param timeout: リクエストを強制終了させるまでのタイムアウト時間。
        複数回に分割リクエストが行われる場合でも、リクエストの合計時間とこの値を比較する。
        また、`timeout` のうち 1/4 を接続時間に、 3/4 を読み込み時間に割り当てている。
        :return: レスポンスオブジェクト
        """
        if self._limit <= 100:
            self._query["_limit"] = str(self._limit)
        else:
            self._query["_limit"] = "100"

        total_time = 0.0

        (response, response_time) = self._request(timeout)
        total_time += response_time

        results: List[SnapshotSearchAPIV2Result] = [response]

        total_count = int(results[0].total_count())

        if total_count <= self._limit:
            self._limit = total_count

        for pos in range(1, math.ceil(self._limit / 100)):
            self._query["_offset"] = str(pos * 100)
            if self._limit < (pos + 1) * 100:
                self._query["_limit"] = str(self._limit % 100)

            time.sleep(response_time)

            while True:
                (response, response_time) = self._request(timeout)
                total_time += response_time
                if total_time > timeout:
                    raise TimeoutError("通信がタイムアウトしました")
                if "meta" in response.json() or response.status() == 200:
                    break
                print("Connection Failed!")
                time.sleep(response_time)
                
            results.append(response)
        return SnapshotSearchAPIV2Result(self._query, results)

    def _request(self, timeout: float) -> Tuple[SnapshotSearchAPIV2Result, float]:
        # `'jsonFilter'` の値までまとめてエンコードされてしまっているので、それを避けるため True にしている
        # TODO: `'q'` がエンコードされずよくないので原因調査して修正する。
        url = self.build_url(True if "jsonFilter" in self._query else False)
        response = requests.get(url, timeout=(timeout / 4, timeout * 3 / 4))
        total_time = response.elapsed.total_seconds()
        tmp = SnapshotSearchAPIV2Result(self._query, response)
        return tmp, total_time

    def build_url(self, decode: bool = True) -> str:
        """
        リクエストオブジェクトに設定されたクエリパラメータから適切な
        URL 文字列を生成する。

        :param decode: True (デフォルト): urlencode を解除する, False: urlencode された文字列を返す
        :return: API リクエストのための URL 文字列
        """
        query = urlencode(self._query)
        if decode:
            query = unquote_plus(query)
        return END_POINT_URL_V2 + '?' + query
